#include <ncurses/curses.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "lyn.h"

void log_config_debug(char *fmt, ...)
{
#if DEBUGLEVEL>1
  FILE *f;
  va_list ap;
  char s[1024];

  va_start(ap,fmt);
  vsprintf(s,fmt,ap);
  va_end(ap);
  if (!strlen(s))
    return;
  f=fopen("lyn-config-debug","a");
  if (!f)
    return;
  fprintf(f,"%d: %s\n",getpid(),s);
  fclose(f);
#endif
}

int config_line(char *ss,struct cline *cc)
{
  signed int i,i1;
  char s[1024],s1[81];

  memset(cc,0,sizeof(struct cline));  
/*  s=(char *)malloc(strlen(ss));
  if (!s) {
    perror("malloc");
    return -1;
  }*/
  /* exclude full-comment lines */
  strcpy(s,ss);
  if (!s[0] || s[0]=='#') {
/*    free(s); */
    return 0;
  }

  /* remove comments */
  for (i=0; i<strlen(s); i++)
/*    if ((s[i]=='#' && s[i-1]!='\\') || s[i]=='\r' || s[i]=='\n'); */
    if (s[i]=='#')
      s[i]=0;
  /* remove trailing whitespace */
  for (i=strlen(s)-1; i>=0; i--)
    if (isspace(s[i]))
      s[i]=0;
    else
      break;
  /* remove heading whitespace */
  while (isspace(s[0]))
    for (i=0; i<strlen(s); i++)
      s[i]=s[i+1];
  if (!s[0]) {
/*    free(s); */
    return 0;
  }
  for (i=0; !isspace(s[i]); i++)
    cc->key[i]=s[i];
  for (; isspace(s[i]); i++)
    ;
  i1=0;
  while (i<strlen(s)) {
    if (i1) {
      strcat(cc->val," ");
      i++;
      i1++;
    }
    for (; !isspace(s[i]) && i<strlen(s); i++,i1++)
      cc->val[i1]=s[i];
  }
  for (i=0; i<strlen(cc->key); i++)
    cc->key[i]=tolower(cc->key[i]);
  log_config_debug("cline: key=%s val=%s",cc->key,cc->val);

/*  free(s); */
  return 0;
  
}

int overlay_config(char *fn, configrec *cfg)
{
  int i,i1;
  char s[81];
  FILE *f;
  struct cline c;

  log_config_debug("overlay_config: appending %s",fn);
  f=fopen(fn,"r");
  if (!f) {
    log_config_debug("overlay_config: couldn't open %s (%s)",
                       fn,sys_errlist[errno]);
    return -1;
  }
  while (fgets(s,160,f)) {
    if (config_line(s,&c))
      continue;
    if (!c.key[0]) {
      continue;
    } else if (!strcmp(c.key,"cdromdevices")) {
      if (cfg->cdromdevices)
        free(cfg->cdromdevices);
      cfg->cdromdevices=(char *)malloc(strlen(c.val));
      strcpy(cfg->cdromdevices,c.val);
      parse_cdrom_devices(cfg);
    } else if (!strcmp(c.key,"verbosity")) {
      cfg->verbosity=atoi(c.val);
    } else if (!strcmp(c.key,"cddbaseloc")) {
      strcpy(cfg->cddbpath,c.val);
    } else if (!strcmp(c.key,"dborder")) {
      strcpy(cfg->dborder,c.val);
    } else if (!strcmp(c.key,"cddbservers")) {
      if (cfg->cddbservers)
        free(cfg->cddbservers);
      cfg->cddbservers=(char *)malloc(strlen(c.val));
      if (cfg->cddbservers)
        strcpy(cfg->cddbservers,c.val);
    } else if (!strcmp(c.key,"sqlcopytolocal")) {
      cfg->sqlcopytolocal=atoi(c.val);
    } else if (!strcmp(c.key,"cddbcopytolocal")) {
      cfg->cddbcopytolocal=atoi(c.val);
    } else if (!strcmp(c.key,"tracksumstart")) {
      if (!strcasecmp(c.val,"songlist"))
        cfg->tracksumstart=_CFG_TRACKSUMSTART_SL;
      else if (!strcasecmp(c.val,"debug"))
        cfg->tracksumstart=_CFG_TRACKSUMSTART_DEBUG;
    } else if (!strcmp(c.key,"whichvolume")) {
      if (!strcasecmp(c.val,"mixer"))
        cfg->whichvolume=_CFG_WHICHVOLUME_MIXER;
      else if (!strcasecmp(c.val,"cdrom"))
        cfg->whichvolume=_CFG_WHICHVOLUME_CD;
      else if (!strcasecmp(c.val,"both"))
        cfg->whichvolume=_CFG_WHICHVOLUME_BOTH;
      else
        cfg->whichvolume=_CFG_WHICHVOLUME_DEFAULT;
    } else if (!strcmp(c.key,"startforcevolume")) {
      if (!strcasecmp(c.val,"off"))
        cfg->forcevolume=-1;
      else if (!strcmp(c.val,"0") || !strcmp(c.val,"0%"))
        cfg->forcevolume=0;
      else if (atoi(c.val))
        cfg->forcevolume=atoi(c.val);
    }
  }
  fclose(f);    
  return 0;
}

void parse_cdrom_devices(configrec *cfg)
{
  char *s,*s2,*ss;
  int f;
  
  if (!cfg->cdromdevices) {
    strcpy(cfg->active_cdromdevice,"/dev/cdrom");
    return;
  }
  s=(char *)malloc(strlen(cfg->cdromdevices));
  if (!s) {
    perror("malloc");
    return;
  }
  strcpy(s,cfg->cdromdevices);
  s2=(char *)malloc(strlen(s));
  if (!s2) {
    perror("malloc");
    free(s);
    return;
  }
  s2[0]=0;
  ss=strtok(s," ");
  while (ss) {
    /* simpler than checking permissions; if we can't open it now,
     * we won't be able to open it later, either.
     */
    if (1 || device_available(ss)) {
      if (strlen(s2))
        strcat(s2," ");
      else
        strcpy(cfg->active_cdromdevice,ss);
      strcat(s2,ss);
    } else
      fprintf(stderr,"%s removed from device list (%s).\n",ss,sys_errlist[errno]);
    ss=strtok(NULL," ");
  }
  strcpy(cfg->cdromdevices,s2);
  free(s);
  free(s2);
}

int read_config(configrec *cfg)
{
  int i,i1;
  char s[81];
  FILE *f;
  
  blank_config();
  cfg->verbosity=_CFG_VERBOSITY_NORMAL;
  cfg->tracksumstart=_CFG_TRACKSUMSTART_NORM;
  cfg->forcevolume=_CFG_FORCEVOLUME_NONE;
  cfg->configfiles=0;
  strcpy(cfg->cddbpath,DEFAULT_LOCAL_LIB);
  cfg->cdromdevices=malloc(strlen(DEFAULT_CD_DEVICE));
  if (!cfg->cdromdevices) {
    perror("malloc");
    return -1;
  }
  strcpy(cfg->cdromdevices,"/dev/cdrom");
  strcpy(cfg->active_cdromdevice,"/dev/cdrom");
  strcpy(cfg->dborder,"local cddb sql");
  cfg->cddbservers=(char *)malloc(strlen(DEFAULT_CDDB_SERVERS));
  strcpy(cfg->cddbservers,DEFAULT_CDDB_SERVERS);
  overlay_config("/usr/local/lib/lyn/system-lynrc",cfg);
  overlay_config("/usr/local/lib/lyn/lynrc",cfg);
  overlay_config("/usr/lib/lyn/lynrc",cfg);
  sprintf(s,"%s/.lynrc",getenv("HOME"));
  overlay_config(s,cfg);
  overlay_config("lynrc",cfg);
/*  overlay_config_env(cfg); */ /* -- later */
  return 0;
}

void blank_config(void)
{
/*   clear_config(0); */
}

void prep_new_config(void)
{
  clear_config(1);
}


void clear_config(int new)
{
  int i;
  
  for (i=0; i<NUM_CONFIGS; i++) {
    if (config.settings[i].name && !new)
      free(config.settings[i].name);
    config.settings[i].name=NULL;
    config.settings[i].type=0;
    if (config.settings[i].cval && !new)
      free(config.settings[i].cval);
    config.settings[i].cval=NULL;
    config.settings[i].ival=0;
    config.settings[i].fval=0;
  }
}

int store_config(char *name, char *value)
{
  int i;
  
  if (!name || strlen(name)==0)
    return -2;
  for (i=0; i<NUM_CONFIGS; i++) {
    if (!config.settings[i].name) {
      if ((config.settings[i].name=(char *)malloc(strlen(name)))==NULL) {
        perror("malloc");
        return -1;
      }
      strcpy(config.settings[i].name,name);
      if (config.settings[i].cval!=NULL)
        if ((config.settings[i].cval=(char *)malloc(strlen(value)))==NULL) {
          perror("malloc");
          return -1;
        }      
      strcpy(config.settings[i].cval,value);
      config.settings[i].cval=NULL;
      config.settings[i].ival=0;
      config.settings[i].fval=0;
      if (atoi(value))
        config.settings[i].ival=atoi(value);
      if (atof(value))
        config.settings[i].fval=atof(value);
    } else if (!strcmp(config.settings[i].name,name)) {
      if (config.settings[i].cval!=NULL)
        if ((config.settings[i].cval=(char *)malloc(strlen(value)))==NULL) {
          perror("malloc");
          return -1;
        }
      strcpy(config.settings[i].cval,value);
      config.settings[i].cval=NULL;
      config.settings[i].ival=0;
      config.settings[i].fval=0;
      if (atoi(value))
        config.settings[i].ival=atoi(value);
      if (atof(value))
        config.settings[i].fval=atof(value);    
    }
  }
}

char *aconfig(char *name)
{
  int i;
  
  for (i=0; i<NUM_CONFIGS; i++) {
    if (!config.settings[i].name)
      return NULL;
    if (!strcmp(name,config.settings[i].name))
      return config.settings[i].cval;
  }
  return NULL;
}

signed int iconfig(char *name)
{
  int i;
  
  for (i=0; i<NUM_CONFIGS; i++) {
    if (!config.settings[i].name)
      return -65535;
    if (!strcmp(name,config.settings[i].name))
      return config.settings[i].ival;
  }
  return -65534;
}
