#include <stdarg.h>
#include <ncurses/curses.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>

#include "lyn.h"

static int hascolor=0;

char color_translation[]={11,14,5,31,2,140,9,1,3};

unsigned char *charstr(int len, unsigned char rc)
{
  static unsigned char s[161];

  if (rc==196)
    rc=ACS_HLINE;
  memset(s,rc,len);
  s[len]=0;
  return(s);
}

void vmode(void)
{
}

int getkey(void)
{
  int ch;

  nodelay(stdscr,TRUE);
  ch=getch();
  nodelay(stdscr,FALSE);
  if (ch==-1)
    return -1;
  ungetch(ch);
  return 1;
}

void cstr(unsigned x, unsigned y, char attrib, unsigned len)
{
  int i;

  switch (attrib) {
    case 1:
    case 2:
    case 3:
    case 4:
    case 5:
    case 6:
    case 7:
      if (hascolor)
        attrset((chtype)COLOR_PAIR(attrib));
      break;
    case 8:
      attroff(A_BOLD);
      if (hascolor)
        attrset((chtype)COLOR_PAIR(8));
      attron(A_DIM);
    case 9:
    case 10:
    case 11:
    case 12:
    case 13:
    case 14:
    case 15:
      if (hascolor)
        attrset(COLOR_PAIR(attrib));
      attron(A_BOLD);
      break;
    case 31:
      if (hascolor) {
        attrset(COLOR_PAIR(attrib));
        attron(A_BOLD);
      } else
        attron(A_REVERSE);
  }
  for (i=0; i<len; i++) {
    move(y-1,x-1+i);
    echochar(ACS_HLINE);
  }
#ifndef SLOPPY_CURSOR
  move(0,0);
#endif
}

void cwrite(unsigned x, unsigned y, char attrib, char *fmt, ...)
{
  char s[512],s1[160],cs[5],attr;
  int i,len,i1,i2,i3=0,cc=0,offset=0;
  va_list ap;

  memset(s,0,160);
  memset(s1,0,160);
  memset(cs,0,5);
  va_start(ap,fmt);
  vsprintf(s,fmt,ap);
  va_end(ap);
  attr=attrib;
  len=strlen(s);
  for (i=0; i<len; i++) {
    if (cc==1) {
      cc=0;
      f_write(x+offset,y,attr,s1);
      offset+=strlen(s1);
      attr=color_translation[(int)s[i]];
    } else if (cc==2) {
      cc=i3=i2=0;
      i1=i;
      while ((s[i1]>='0') &&(s[i1]<='9'))
        cs[i2++]=s[i1++];
      cs[i2]=s[i]=0;
      i+=i2-1;
      f_write(x+offset,y,attr,s1);
      offset+=strlen(s1);
      attr=atoi(cs);
    } else if ((s[i]=='|') &&(s[i+1]>='0') &&
              (s[i+1]<='9') &&(s[i-1]!='\\'))
      cc=2;
    else if (s[i]==3)
      cc=1;
    else
      s1[i3++]=s[i];
  }
  s1[i3]=0;
  f_write(x+offset,y,attr,s1);
}

void dwrite(unsigned x, unsigned y, char attrib, char *fmt, ...)
{
  va_list ap;
  char s[768];
/*  char *s; */

/*  s=(char *)malloc(strlen(fmt)*3);*/ 
  va_start(ap,fmt);
  vsprintf(s,fmt,ap);
  va_end(ap);
  f_write(x,y,attrib,s);
/*  free(s);  */
}

void cnwrite(int y, char attrib, char *fmt, ...)
{
  va_list ap;
/*   char s[512]; */
  char *s;

  s=(char *)malloc(strlen(fmt)*3);
  va_start(ap,fmt);
  vsprintf(s,fmt,ap);
  va_end(ap);
  f_write((80-strlen(s))/2,y,attrib,s);
  free(s);
}

void init_curses(void)
{
  initscr();
  hascolor=has_colors(); 
  if (hascolor) {
    start_color();

    init_pair(1, COLOR_BLUE, COLOR_BLACK);
    init_pair(2, COLOR_GREEN, COLOR_BLACK);
    init_pair(3, COLOR_CYAN, COLOR_BLACK);
    init_pair(4, COLOR_RED, COLOR_BLACK);
    init_pair(5, COLOR_MAGENTA, COLOR_BLACK);
    init_pair(6, COLOR_YELLOW, COLOR_BLACK);
    init_pair(7, COLOR_WHITE, COLOR_BLACK);
    init_pair(8, COLOR_BLACK, COLOR_BLACK);
    init_pair(9, COLOR_BLUE, COLOR_BLACK);
    init_pair(10, COLOR_GREEN, COLOR_BLACK);
    init_pair(11, COLOR_CYAN, COLOR_BLACK);
    init_pair(12, COLOR_RED, COLOR_BLACK);
    init_pair(13, COLOR_MAGENTA, COLOR_BLACK);
    init_pair(14, COLOR_YELLOW, COLOR_BLACK);
    init_pair(15, COLOR_WHITE, COLOR_BLACK);
    init_pair(31,COLOR_WHITE, COLOR_BLUE);
    attrset(COLOR_PAIR(8));
  }
  cbreak();
  nonl();
  noecho();
  erase();
  nodelay(stdscr,TRUE);
  keypad(stdscr,TRUE);
}

void f_write(unsigned x, unsigned y, char attrib, char *str)
{
  switch (attrib) {
    case 1:
    case 2:
    case 3:
    case 4:
    case 5:
    case 6:
    case 7:
      if (hascolor)
        attrset((chtype)COLOR_PAIR(attrib));
      break;
    case 8:
      attroff(A_BOLD);
      if (hascolor)
        attrset((chtype)COLOR_PAIR(8));
      attron(A_DIM);
    case 9:
    case 10:
    case 11:
    case 12:
    case 13:
    case 14:
    case 15:
      if (hascolor)
        attrset(COLOR_PAIR(attrib));
      attron(A_BOLD);
      break;
    case 31:
      if (hascolor) {
        attrset(COLOR_PAIR(attrib));
        attron(A_BOLD);
      } else
        attron(A_REVERSE);
  }
    
  move(y-1,x-1);
  addstr(str);
  standend();
  if (attrib==31) {
    wattroff(stdscr,A_BOLD);
  }
  /*
  attroff(A_REVERSE);
  attroff(A_BOLD);
  attroff(A_DIM);*/
#ifndef SLOPPY_CURSOR
  move(0,0);
#endif
}

void sig_winch_handler(int snum)
{
  /* Handle SIGWINCH for xterm resizes */
  endwin();
  init_curses();
  redraw_screen();
  signal(SIGWINCH,sig_winch_handler);
}

